package com.emarcotte.testrio;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.media.opengl.GL;

import com.emarcotte.glfoo.utils.IndexedVertexBufferObject;
import com.sun.opengl.util.texture.Texture;

public class MD5Model {
	MD5Frame bindpos;
	MD5Frame currentPose;
	MD5Frame currentFrame;
	List<MD5Animation> anims;

	List<MD5Mesh> meshes;
	int anim;
	Texture t;

	public MD5Model(int numJoints, int numMeshes) {
		bindpos = new MD5Frame(numJoints);
		meshes = new ArrayList<MD5Mesh>(numMeshes);
		anims = new ArrayList<MD5Animation>();
	}

	public void computeBindPosition() {
		for (MD5Mesh m : meshes) {
			m.update(bindpos);
		}
	}

	public void addAnimation(MD5Animation currentAnimation) {
		anims.add(currentAnimation);
	}

	public void addMesh(int i, MD5Mesh currentMesh) {
		meshes.add(i, currentMesh);
	}

	public void setBindPos() {
		currentFrame = bindpos;
	}

	public void updateAnimation() {
		currentFrame = anims.get(anim).getCurrentFrame();

		MD5Frame nextFrame = anims.get(anim).getNextFrame();

		// TODO: Do NOT USE BIND POS!! Should copy it and use some sort of
		// "currentState"
		bindpos.interpolate(currentFrame, nextFrame, anims.get(anim)
				.getFactor());

		for (MD5Mesh m : meshes) {
			m.update(bindpos);
			m.computeNormals(bindpos);
		}
	}

	public List<IndexedVertexBufferObject> getMeshVBOs() {
		LinkedList<IndexedVertexBufferObject> vbos = new LinkedList<IndexedVertexBufferObject>();

		for (MD5Mesh m : meshes) {

			IndexedVertexBufferObject vbo = new IndexedVertexBufferObject(m
					.getIndices());

			vbo.setVertexPositions(m.getPositions(), GL.GL_TRIANGLES);
			vbo.setTextureCoordinates(m.getTexCoords(), 2);
			vbo.setNormalData(m.getNormals());

			vbos.add(vbo);
		}
		return vbos;
	}

	public void setBaseFrame(MD5Frame joints) {
		bindpos = joints;
	}

	public void nextAnim() {
		anim++;
		if (anim >= anims.size()) {
			anim = 0;
		}

	}

	public void drawBones(GL gl) {
		gl.glBegin(GL.GL_LINES);
		for (MD5Joint j : bindpos.getJoints()) {
			if (j.hasParent()) {
				gl.glVertex3d(j.getPos().i, j.getPos().j, j.getPos().k);
				gl.glVertex3d(j.getParent().getPos().i,
						j.getParent().getPos().j, j.getParent().getPos().k);
			}
		}
		gl.glEnd();
	}

	public List<MD5Mesh> getMeshes() {
		return meshes;
	}

	public void computeNormals() {
		for (MD5Mesh mesh : meshes) {
			mesh.computeNormals(bindpos);
		}

	}

}
