package com.emarcotte.testrio;

import java.util.List;

import com.emarcotte.glfoo.math.Vector3;

/**
 * MD5 ModelLoader UVCoordinate. Deals with weights and texture mapping
 * primarily.
 * 
 * @author Eugene Marcotte
 */

public class MD5Vertex {
	private int index;
	private int firstWeight;
	private int numWeights;
	private Vector3 wv;

	/**
	 * Create a vertex.
	 */
	public MD5Vertex(int index, int weightIndex, int weightElem) {
		this.firstWeight = weightIndex;
		this.numWeights = weightElem;
		this.index = index;
		wv = new Vector3();
	}

	public void recalculatePosition(MD5Mesh m, List<MD5Weight> weights,
			MD5Frame joints) {

		double x = 0, y = 0, z = 0;

		// calculate final vertex to draw with weights
		for (int i = 0; i < numWeights; ++i) {
			MD5Weight w = weights.get(firstWeight + i);
			MD5Joint j = joints.getJoint(w.getJoint());
			double bias = w.getBias();

			// calculate transformed vertex for this weight
			j.getOrientation().rotatePoint(w.getPosition(), wv);

			// the sum of all weight->bias should be 1.0
			Vector3 jointPos = j.getPos();

			x += ((jointPos.getI() + wv.i) * bias);
			y += ((jointPos.getJ() + wv.j) * bias);
			z += ((jointPos.getK() + wv.k) * bias);

		}
		m.setVertData(index, x, y, z);
	}

	public int getId() {
		return index;
	}

	public int getWeightCount() {
		return numWeights;
	}

	public int getStartWeight() {
		return firstWeight;
	}
}
