package modelview;

import java.io.IOException;
import java.io.InputStream;

import javax.media.opengl.GL;
import javax.media.opengl.GLException;

import com.emarcotte.glfoo.GraphicsDriver;
import com.emarcotte.glfoo.Initializable;
import com.emarcotte.glfoo.Renderable;
import com.emarcotte.glfoo.utils.ResourceHandler;
import com.sun.opengl.util.texture.Texture;
import com.sun.opengl.util.texture.TextureIO;

public class CubeEnvironment implements Renderable, Initializable {
	Texture negativex;
	Texture negativey;
	Texture negativez;
	Texture positivex;
	Texture positivey;
	Texture positivez;

	@Override
	public void draw(GraphicsDriver driver) {

		GL gl = driver.getGl();
		gl.glPushAttrib(GL.GL_LIGHTING_BIT);
		gl.glDisable(GL.GL_LIGHTING);
		gl.glColor3d(1, 1, 1);
		gl.glPushMatrix();
		gl.glScaled(300, 300, 300);

		negativex.bind();
		gl.glBegin(GL.GL_QUADS);
		gl.glTexCoord2d(0, 1);
		gl.glVertex3d(-1, -1, 1);
		gl.glTexCoord2d(0, 0);
		gl.glVertex3d(-1, 1, 1);
		gl.glTexCoord2d(1, 0);
		gl.glVertex3d(-1, 1, -1);
		gl.glTexCoord2d(1, 1);
		gl.glVertex3d(-1, -1, -1);
		gl.glEnd();

		negativey.bind();
		gl.glBegin(GL.GL_QUADS);
		gl.glTexCoord2d(1, 1);
		gl.glVertex3d(1, -1, 1);
		gl.glTexCoord2d(1, 0);
		gl.glVertex3d(-1, -1, 1);
		gl.glTexCoord2d(0, 0);
		gl.glVertex3d(-1, -1, -1);
		gl.glTexCoord2d(0, 1);
		gl.glVertex3d(1, -1, -1);
		gl.glEnd();

		negativez.bind();
		gl.glBegin(GL.GL_QUADS);
		gl.glTexCoord2d(0, 1);
		gl.glVertex3d(-1, -1, -1);
		gl.glTexCoord2d(0, 0);
		gl.glVertex3d(-1, 1, -1);
		gl.glTexCoord2d(1, 0);
		gl.glVertex3d(1, 1, -1);
		gl.glTexCoord2d(1, 1);
		gl.glVertex3d(1, -1, -1);
		gl.glEnd();

		positivex.bind();
		gl.glBegin(GL.GL_QUADS);
		gl.glTexCoord2d(1, 0);
		gl.glVertex3d(1, 1, 1);
		gl.glTexCoord2d(1, 1);
		gl.glVertex3d(1, -1, 1);
		gl.glTexCoord2d(0, 1);
		gl.glVertex3d(1, -1, -1);
		gl.glTexCoord2d(0, 0);
		gl.glVertex3d(1, 1, -1);
		gl.glEnd();

		positivey.bind();
		gl.glBegin(GL.GL_QUADS);
		gl.glTexCoord2d(0, 1);
		gl.glVertex3d(-1, 1, -1);
		gl.glTexCoord2d(0, 0);
		gl.glVertex3d(-1, 1, 1);
		gl.glTexCoord2d(1, 0);
		gl.glVertex3d(1, 1, 1);
		gl.glTexCoord2d(1, 1);
		gl.glVertex3d(1, 1, -1);
		gl.glEnd();

		positivez.bind();
		gl.glBegin(GL.GL_QUADS);
		gl.glTexCoord2d(1, 0);
		gl.glVertex3d(-1, 1, 1);
		gl.glTexCoord2d(1, 1);
		gl.glVertex3d(-1, -1, 1);
		gl.glTexCoord2d(0, 1);
		gl.glVertex3d(1, -1, 1);
		gl.glTexCoord2d(0, 0);
		gl.glVertex3d(1, 1, 1);

		gl.glEnd();
		gl.glBindTexture(GL.GL_TEXTURE_2D, 0);
		gl.glPopMatrix();
		gl.glPopAttrib();
	}

	private InputStream getStream(String p) {
		ResourceHandler handler = ResourceHandler.getHandler();
		return handler.getResourceAsStream(p);
	}

	Texture loadTexture(String path, String ext) {
		Texture t = null;
		try {
			t = TextureIO.newTexture(getStream(path), false, ext);
		} catch (GLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return t;
	}

	@Override
	public void init(GraphicsDriver driver) {
		negativex = loadTexture("textures/autumn_negative_x.png", TextureIO.PNG);
		negativey = loadTexture("textures/autumn_negative_y.png", TextureIO.PNG);
		negativez = loadTexture("textures/autumn_negative_z.png", TextureIO.PNG);
		positivex = loadTexture("textures/autumn_positive_x.png", TextureIO.PNG);
		positivey = loadTexture("textures/autumn_positive_y.png", TextureIO.PNG);
		positivez = loadTexture("textures/autumn_positive_z.png", TextureIO.PNG);
	}
}
