package modelview;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;

import com.emarcotte.glfoo.ApplicationContext;
import com.emarcotte.glfoo.GraphicsDriver;
import com.emarcotte.glfoo.Initializable;
import com.emarcotte.glfoo.Renderable;
import com.emarcotte.glfoo.RenderingContext;
import com.emarcotte.glfoo.implementation.ApplicationContextProvider;
import com.emarcotte.glfoo.implementation.DefaultRenderingContext;
import com.emarcotte.glfoo.implementation.FocusableCamera;
import com.emarcotte.glfoo.implementation.GraphicRenderer;
import com.emarcotte.glfoo.utils.GLShader;
import com.emarcotte.glfoo.utils.GLShaderProgram;
import com.emarcotte.glfoo.utils.IndexedVertexBufferObject;
import com.emarcotte.glfoo.utils.Light2;
import com.emarcotte.glfoo.utils.ResourceHandler;
import com.emarcotte.glfoo.utils.VertexBufferObject;
import com.emarcotte.testrio.MD5Loader;
import com.emarcotte.testrio.MD5Mesh;
import com.emarcotte.testrio.MD5Model;
import com.sun.opengl.util.texture.Texture;
import com.sun.opengl.util.texture.TextureIO;

public class Md5Test extends DefaultRenderingContext implements Renderable,
		Initializable {

	Texture normal;
	Texture regular;
	private GLShaderProgram program;
	private List<IndexedVertexBufferObject> vbos;
	MD5Model m;
	Light2 l;
	Prefs prefs;

	private class Prefs extends KeyAdapter {
		boolean wireframe = false;
		boolean normals = false;
		boolean animate = false;
		boolean bones = false;
		boolean shader = false;

		@Override
		public void keyPressed(KeyEvent e) {
			if (e.getKeyCode() == KeyEvent.VK_W) {
				wireframe = !wireframe;
			} else if (e.getKeyCode() == KeyEvent.VK_N) {
				normals = !normals;
			} else if (e.getKeyCode() == KeyEvent.VK_B) {
				bones = !bones;
			} else if (e.getKeyCode() == KeyEvent.VK_A) {
				animate = !animate;
			} else if (e.getKeyCode() == KeyEvent.VK_S) {
				shader = !shader;
			}
		}
	}

	public Md5Test(ApplicationContext context) {
		FocusableCamera dc = new FocusableCamera();
		CubeEnvironment cube = new CubeEnvironment();
		dc.setFocusY(50);
		dc.setDistance(50);
		GraphicRenderer renderer = new GraphicRenderer(dc);
		addRenderer(renderer);
		renderer.addRenderable(cube);
		renderer.addRenderable(this);
		context.getGraphicsDriver().addInitializable(this);
		context.getGraphicsDriver().addInitializable(cube);
		context.getMainWindow().addMouseHandler(new TestrioMouseHandler(dc));

		prefs = new Prefs();
		context.getMainWindow().addKeyListener(prefs);

		loadModelStuff();
	}

	/**
	 * Just loads models ugly nasty brute force to get things on the screen
	 */
	private void loadModelStuff() {
		MD5Loader loader = new MD5Loader();

		InputStream mesh = ResourceHandler.getHandler().getResourceAsStream(
				"boney/boney.md5mesh");

		Map<String, InputStream> animations;
		animations = new HashMap<String, InputStream>();
		InputStream anim = ResourceHandler.getHandler().getResourceAsStream(
				"boney/chest_pain1.md5anim");
		animations.put("boney/chest_pain1.md5anim", anim);

		loader.loadModel(mesh, animations);

		m = loader.getModel();
		vbos = m.getMeshVBOs();
	}

	@Override
	public void init(GraphicsDriver driver) {
		GL gl = driver.getGl();
		GLU glu = driver.getGlu();

		// load textures
		try {
			normal = TextureIO.newTexture(ResourceHandler.getHandler()
					.getResourceAsStream("normals.png"), true, TextureIO.JPG);
			regular = TextureIO.newTexture(ResourceHandler.getHandler()
					.getResourceAsStream("tex.png"), true, TextureIO.JPG);
		} catch (Exception e) {
			e.printStackTrace();
		}

		// remove back faces
		gl.glEnable(GL.GL_CULL_FACE);
		gl.glFrontFace(GL.GL_CW);
		gl.glCullFace(GL.GL_BACK);

		// set up a light
		l = new Light2();
		// l.setDirectional(true);
		l.setPosition(0, 100, 0);
		l.setAmbientColor(.5f, .5f, .5f);
		l.setDiffuseColor(.5f, .5f, .5f);
		l.setSpecularColor(1, 0, 0);
		l.applyLight(GL.GL_LIGHT0, gl);

		gl.glEnable(GL.GL_LIGHTING); // not needed shaders are running.
		gl.glEnable(GL.GL_LIGHT0); // not needed shaders are running.
		gl.glEnable(GL.GL_TEXTURE_2D); // probably not needed shaders are
		// running.

		// set up the shader stuff
		program = GLShaderProgram.createProgram(gl);

		GLShader vertexShader = GLShader.load(gl, glu,
				"shaders/phong_vert.glsl", GL.GL_VERTEX_SHADER);

		GLShader fragShader = GLShader.load(gl, glu, "shaders/phong_frag.glsl",
				GL.GL_FRAGMENT_SHADER);

		program.add(vertexShader);
		program.add(fragShader);
		program.link();
		program.enable();
		// set uniforms for the shader
		program.setUniformi("colorMap", 0);
		program.setUniformi("normalMap", 1);
		program.disable();

		for (IndexedVertexBufferObject v : vbos) {
			v.init(driver);
		}
	}

	public void draw(GraphicsDriver driver) {
		GL gl = driver.getGl();
		// gl.glClearColor(0, 0, 0, 0);
		gl.glColor3d(1, 1, 1);
		// setup multi texturing and bind textures.
		gl.glActiveTexture(GL.GL_TEXTURE0);
		regular.bind();
		gl.glActiveTexture(GL.GL_TEXTURE1);
		normal.bind();

		// driver.getGlut().glutSolidCube(arg0)
		l.applyLight(GL.GL_LIGHT0, gl);

		if (prefs.animate)
			m.updateAnimation();
		else
			m.setBindPos();

		gl.glPushMatrix();
		gl.glRotated(90, 0, 0, -1);
		gl.glRotated(90, 0, -1, 0);
		gl.glPushAttrib(GL.GL_POLYGON_BIT);

		if (prefs.wireframe)
			gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_LINE);

		if (prefs.shader) {
			// use special shader and draw
			program.enable();
		}
		gl.glPushMatrix();
		for (VertexBufferObject v : vbos) {
			v.draw(driver);
			gl.glTranslated(50, 0, 0);
		}

		gl.glPopMatrix();
		gl.glPopAttrib();
		if (prefs.shader) {
			program.disable();
		}

		// set textures back to nothing
		gl.glBindTexture(regular.getTarget(), 0);
		gl.glActiveTexture(GL.GL_TEXTURE0);
		gl.glBindTexture(normal.getTarget(), 0);

		// draw normals?
		if (prefs.normals || prefs.bones) {

			gl.glPushAttrib(GL.GL_LIGHTING_BIT);
			gl.glEnable(GL.GL_COLOR_MATERIAL);
			gl.glPushAttrib(GL.GL_CURRENT_BIT);

			for (MD5Mesh mesh : m.getMeshes()) {
				if (prefs.normals) {
					gl.glColor3d(1, 0, 0);

					mesh.drawNormals(gl);
				}

				if (prefs.bones) {
					gl.glPushAttrib(GL.GL_DEPTH_BUFFER_BIT);
					gl.glDisable(GL.GL_DEPTH_TEST);
					gl.glColor3d(0, 1, 0);
					m.drawBones(gl);
					gl.glPopAttrib();
				}

				gl.glTranslated(50, 0, 0);
			}
			gl.glPopAttrib();
			gl.glPopAttrib();
		}

		gl.glPopMatrix();

	}

	public static void main(String[] args) {
		ApplicationContext context = ApplicationContextProvider.getInstance()
				.getContext();
		context.run();
		RenderingContext mm = new Md5Test(context);
		context.setCurrentRenderingContext(mm);

	}
}
